; #################################################################################### 
; ####################################################################################

    Page_Directory_Address      equ 0C0300000h
    Page_Directory_Address_PAE  equ 0C0600000h
    Page_Table_Address          equ 0C0000000h
    
    cr4_flag_PAE                equ (1 SHL 5)
   
    Present_Flag                equ 1             ; 1=present;                0=not present
    PCD_Flag                    equ (1 SHL 4)     ; 1=page non mise en cache  0=page mise en cache
    RW_Flag                     equ (1 SHL 1)     ; 1=ReadWrite;              0=Read only
    US_Flag                     equ (1 SHL 2)     ; 1=user;                   0=supervisor                      
    PageSize_Flag               equ (1 SHL 7)     ; 1=4Mbyte;                 0=4Kbyte (non PAE)
                                                  ; 1=2Mbyte;                 0=4Kbyte (PAE)
    

    cr0_flag_write_protect      equ (1 SHL 16)    ; 1=Non autorisation d'crire du Supervisor-Level dans
                                                  ; une page read only.
                                                  ; 0=Autorisation d'crire du Supervisor-Level dans
                                                  ; une page read only.                                               
    
; #################################################################################### 
; ####################################################################################

GetPageEntry MACRO

local sortie, non_pae, non_pae_2, invalide

    push ebx
    push ecx

    mov eax, dword ptr [esp+4+(2*4)]      ; adresse virtuelle         
    mov ebx, eax 
 
    xor edx,edx
    shr eax, 22     ; Index Page Directory
    shl eax, 2      ; mul par 4
    add eax, Page_Directory_Address
 
    mov ecx, cr4
    test ecx, cr4_flag_PAE 
    je non_pae
    
        mov eax,ebx
        shr eax, 21     ; Index Page Directory
        shl eax, 3      ; mul par 8
        add eax, Page_Directory_Address_PAE
        mov edx, dword ptr [eax+4] 
                        
non_pae :       
    mov eax, dword ptr [eax]          ; Directory Entry
    
    test al, Present_Flag
    je invalide   
    
    test al, PageSize_Flag
    jne sortie                        ; saute si les pages sont de 4 MByte (non PAE) ou 2Mbyte (PAE)   
    
    ; page 4KByte
    xor edx, edx        
    shr ebx, 12
    shl ebx,2 
    
    test ecx, cr4_flag_PAE 
    je non_pae_2  
        shl ebx,1
        mov edx, dword ptr [ebx+Page_Table_Address+4] 
non_pae_2 :    
    mov eax, dword ptr [ebx+Page_Table_Address]        
sortie :    
    pop ecx
    pop ebx
    ret 4
invalide :
    xor eax,eax
    xor edx,edx
    jmp sortie   
ENDM   

; ------------------------------------------------------------------------------------ 

get_page_entry :
    GetPageEntry  
    
; ------------------------------------------------------------------------------------     
    
; #################################################################################### 
; ####################################################################################

IsVirtualAddressValid MACRO

local sortie

    push ebx
    push edx
    xor ebx,ebx

        mov eax, dword ptr [esp+4+(2*4)]
        
        push eax
        call get_page_entry
        
        test al, Present_Flag
        je sortie
                
        inc ebx  
sortie :
    mov eax,ebx
   
    pop edx
    pop ebx
    ret 4
ENDM

; ------------------------------------------------------------------------------------ 
is_virtual_address_valid :
    IsVirtualAddressValid      
; ------------------------------------------------------------------------------------ 
 
; #################################################################################### 
; ####################################################################################

IsVirtualAddressValidAndCache MACRO

local sortie

    push ebx
    push edx
    xor ebx,ebx

        mov eax, dword ptr [esp+4+(2*4)]
        
        push eax
        call get_page_entry
        
        test al, Present_Flag
        je sortie        
        test al, PCD_Flag
        jne sortie
                
        inc ebx  
sortie :
    mov eax,ebx
   
    pop edx
    pop ebx
    ret 4
ENDM

; ------------------------------------------------------------------------------------ 
is_virtual_address_valid_and_cache :
    IsVirtualAddressValidAndCache      
; ------------------------------------------------------------------------------------  
 
; #################################################################################### 
; ####################################################################################

IsWriteVirtualAddress MACRO

local sortie, ok

    push ebx
    push ecx
    push edx
    
    xor ebx,ebx   
        
        mov eax, dword ptr [esp+4+(3*4)]      
        
        push eax    
        call get_page_entry 
                         
        test al, Present_Flag
        je sortie 
                
        test al, RW_Flag             
        jne ok  
        ; page Read Only 
        mov ecx, cr0
        test ecx, cr0_flag_write_protect      
        jne sortie   
ok :         
    inc ebx       ; ok :)              
sortie :
    mov eax, ebx
    
    pop edx
    pop ecx
    pop ebx
    ret 4   

ENDM  
     
; ------------------------------------------------------------------------------------ 
is_write_virtual_address :
    IsWriteVirtualAddress      
; ------------------------------------------------------------------------------------ 

; #################################################################################### 
; ####################################################################################

